<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\Session\Session;
use Zeedhi\Framework\DTO\Response\Message;
use \Setforms2\Shared\ORM\DateTime;

use Teknisa\MAN\Utilities\Util;
use Teknisa\MAN\Services\ParamsProduto as ParamsProdutoService;


class TabelaPreco extends \Teknisa\MAN\Controllers\TeknisaCrud {
    
    protected $dataSourceManager;
    protected $dataSourceName = 'vnd42400_tabelapreco';
    protected $environment;
    protected $crudService;
    protected $TabelaPrecoService;
    protected $paramsProdutoService;
    protected $utilService;
    private   $calcularPrecosService;
    
    public static $logMessages = array();
    
    public function __construct($dataSourceManager, $crudService, $environment, $TabelaPrecoService, ParamsProdutoService $paramsProdutoService, Util $utilService, $calcularPrecosService) {
        $this->dataSourceManager     = $dataSourceManager;
        $this->crudService           = $crudService;
        $this->environment           = $environment;
        $this->TabelaPrecoService    = $TabelaPrecoService;
        $this->paramsProdutoService  = $paramsProdutoService;
        $this->util                  = $utilService;
        $this->calcularPrecosService = $calcularPrecosService;
    }
    
    public static function escreverLog ($data) {
        array_push(self::$logMessages, $data);
    }
    
    public static function escreverLogArray($array) {
        foreach ($array as $linha) {
            self::escreverLog($linha);
        }
    }
    
    public static function getLog () {
        return self::$logMessages;
    }
    
    public function findProdutosNota(DTO\Request $request, DTO\Response $response){
        $req = $this->util->getParamsAsArray($request);
        $this->parameterBagColumns = array('NRLANCTONF', 'CDFILIAL', 'NRORG', 'CDTABEPREC', 'ULTCOMPRA');
        if ($req['ULTCOMPRA'] == 'S') {
            $this->dataSourceName = "produtosTabelaPreco";
            parent::find($request,$response);
        } else {
            $this->dataSourceName = "produtosNota";
            parent::find($request,$response);
        }
    }
    
    public function findNotaFiscal(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = "select_nota_by_filial";
        parent::find($request,$response);
    }

    public function findPrecoDiferenciado(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = "vnd42400_tabelapreco_precodiferenciado";
        parent::find($request,$response);
    }
    
    public function deleteVigencia(DTO\Request $request, DTO\Response $response){
        $row = $request->getDataSet()->getRows();
        $CDFILIAL        = $row[0]['CDFILIAL'];
        $CDTABEPREC      = $row[0]['CDTABEPREC'];
        $DTINIVGPREC     = $row[0]['DTINIVGPREC'];
        $DTFINVGPREC     = $row[0]['DTFINVGPREC'];
        $NRORG           = $this->environment->getNrOrgTrab();
        $isParameterized = $this->TabelaPrecoService->isParameterizedBySomething($CDFILIAL,$CDTABEPREC);
        if($isParameterized == 1){
            $response->setError(new DTO\Response\Error('Esta Tabela de Preço está parametrizada por Cliente Filial.',1));
        } elseif($isParameterized == 2){
            $response->setError(new DTO\Response\Error('Esta Tabela de Preço está parametrizada por Loja.',2));
        } elseif($isParameterized == 3){
            $response->setError(new DTO\Response\Error('Esta Tabela de Preço está parametrizada por Filial.',3));
        } elseif($isParameterized == 4){
            $response->setError(new DTO\Response\Error('Esta Tabela de Preço está parametrizada por Cliente.',4));
        } else{
            $this->TabelaPrecoService->deleteVigencia($CDFILIAL,$NRORG,$CDTABEPREC,$DTINIVGPREC);
        }
        if($DTINIVGPREC == null && $DTFINVGPREC == null){
            $this->deleteTabelaPreco($request,$response);
        }
    }
    
    public function findProduto(DTO\Request $request, DTO\Response $response) {
        $req = $this->util->getParamsAsArray($request);
        // Nao remover comentario abaixo
        //if (count($req) > 0 && isset($req['CDFILIAL'])) {
            //$codGrupoProd = $this->TabelaPrecoService->getCdGrupoProd($req['CDFILIAL'], $req['NRORG']);//CDFILIAL;
            //$cdSegmer = $this->TabelaPrecoService->getCdSegmer($req['CDFILIAL'], $req['NRORG']);
            //if($codGrupoProd !== null) {
               //// Utiliza família
                //$nrConfigCount = $this->TabelaPrecoService->getNrConfig($req['CDFILIAL'], $req['NRORG'],$codGrupoProd);
                //$request->getFilterCriteria()->addCondition('CDFILISUBGR', $nrConfigCount[0]['NRCONFIG'] > 0 ? $request[0]['value'] : 'P');
                //if($cdSegmer !== null) {
                    //// Utiliza família / segmento
                    ////  + CDFILISUBGR       /       CDSEGMER
                    //$this->dataSourceName = "vnd42400_tabelapreco_produto_segfam_com_preco";
                    //$request->getFilterCriteria()->addCondition('CDSEGMER', $cdSegmer); //  SEM PREÇO 
                //} else {
                    //// Utiliza família
                    //$this->dataSourceName = "vnd42400_tabelapreco_produto_familia_com_preco";
                //}
            //} else {
                //if($cdSegmer != "") {
                    //// Utiliza segmento
                    ////  +  CDSEGMER
                    //$this->dataSourceName = "vnd42400_tabelapreco_produto_segmento_com_preco";
                    //$request->getFilterCriteria()->addCondition('CDSEGMER', $cdSegmer);
                //}else {
                    //// Não utiliza nada
                    //$this->dataSourceName = "vnd42400_tabelapreco_produto_normal_com_preco";
                    //$CDNVPRODUTO = $this->paramsProdutoService->getCDNVPRODUTO($req['NRORG'])[0]['CDNVPRODUTO'];
                    //$request->getFilterCriteria()->addCondition('CDNVPRODUTO', $CDNVPRODUTO);
                //}
            //}
        //}// COM PREÇO
        
        $this->dataSourceName = "vnd42400_tabelapreco_produto_normal_com_preco";
        $CDNVPRODUTO = $this->paramsProdutoService->getCDNVPRODUTO($req['NRORG'])[0]['CDNVPRODUTO'];
        $request->getFilterCriteria()->addCondition('CDNVPRODUTO', $CDNVPRODUTO);
        
        if(isset($req['CDFILIAL'])) {
            parent::find($request,$response);
        }
    }    
    
    public function findProdutoToPrice(DTO\Request $request, DTO\Response $response){
        $req = $this->util->getParamsAsArray($request);
        // Nao remover comentario abaixo
        //if (count($req) > 0) {
            //$codGrupoProd = $this->TabelaPrecoService->getCdGrupoProd($req['CDFILIAL'], $req['NRORG']);//CDFILIAL;
            //$cdSegmer = $this->TabelaPrecoService->getCdSegmer($req['CDFILIAL'], $req['NRORG']);
            //if($codGrupoProd !== null) {
               //// Utiliza família
                //$nrConfigCount = $this->TabelaPrecoService->getNrConfig($req['CDFILIAL'], $req['NRORG'],$codGrupoProd);
                //$request->getFilterCriteria()->addCondition('CDFILISUBGR',$nrConfigCount[0]['NRCONFIG'] > 0 ? $req['CDFILIAL'] : 'P');
                //if($cdSegmer !== null) {
                  //// Utiliza família / segmento
                    ////  + CDFILISUBGR       /       CDSEGMER
                    //$this->dataSourceName = "vnd42400_tabelapreco_produto_segfam";
                    //$request->getFilterCriteria()->addCondition('CDSEGMER', $cdSegmer); //  SEM PREÇO
                //} else {
                  //// Utiliza família
                    //$this->dataSourceName = "vnd42400_tabelapreco_produto_familia";
                //}
            //} else {
                //if($cdSegmer != "") {
                  //// Utiliza segmento
                    ////  +  CDSEGMER
                    //$this->dataSourceName = "vnd42400_tabelapreco_produto_segmento";
                    //$request->getFilterCriteria()->addCondition('CDSEGMER', $cdSegmer);
                //} else {
                  //// Não utiliza nada
                    //$this->dataSourceName = "vnd42400_tabelapreco_produto_normal";              
                    //$CDNVPRODUTO = $this->paramsProdutoService->getCDNVPRODUTO($req['NRORG'])[0]['CDNVPRODUTO'];
                    //$request->getFilterCriteria()->addCondition('CDNVPRODUTO', $CDNVPRODUTO);
                //}
            //}
        //}
        $this->dataSourceName = "vnd42400_tabelapreco_produto_normal";              
        $CDNVPRODUTO = $this->paramsProdutoService->getCDNVPRODUTO($req['NRORG'])[0]['CDNVPRODUTO'];
        $request->getFilterCriteria()->addCondition('CDNVPRODUTO', $CDNVPRODUTO);
        $filterCriteria = $request->getFilterCriteria();
        $filterCriteria->setDataSourceName($this->dataSourceName);
        $filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
        $newFilterCriteria = $this->crudService->getPopulatedFilterCriteria($filterCriteria, $this->parameterBagColumns);

        parent::find($request,$response);
        
    }    
    
    public function findComposicaoProduto(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = "vnd42400_tabelapreco_composicaoproduto";
        parent::find($request,$response);
    }
    
    public function findVigencias(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = "vnd42400_tabelapreco_vigencias";
        parent::find($request,$response);
    }
    
    public function findVigenciasForReport(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = "vnd42400_tabelapreco_vigencias_report";
        parent::find($request,$response);
    }
    
    public function findVigenciasImport(DTO\Request $request, DTO\Response $response){
        try {
            $row = $request->getFilterCriteria()->getConditions();
            $Res = $this->TabelaPrecoService->findVigenciasImport($row);
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('vnd42400_tabelapreco_vigencias_import', $Res));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }
    
    public function validImportPriceChart(DTO\Request $request, DTO\Response $response){
        try {
            $row = $request->getFilterCriteria()->getConditions();
            $Res = $this->TabelaPrecoService->validImportPriceChart($row);
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('vnd42400_validImportPriceChart', $Res));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function importItensVigencia(DTO\Request $request, DTO\Response $response){
        try {
            
            $row = $request->getFilterCriteria()->getConditions();
            $this->TabelaPrecoService->importItensVigencia($row);
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function findMercado(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = "vnd42400_tabelapreco_mercado";
        parent::find($request,$response);
    }
    
    public function saveTabelaPreco(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = "vnd42400_tabelapreco";
        $row = $request->getDataSet()->getRows();
        
        $currentTime = new \DateTime(null,new \DateTimeZone('America/Sao_Paulo'));
        $isNew = $row[0]['__is_new'];
        $IDATIVO        = 'S';
        if($isNew){
            $CDOPERINCLUSAO = $this->environment->getCdOperador();
            $DTINCLUSAO     = $currentTime->format("d/m/Y");
            $NRORGINCLUSAO  = $this->environment->getNrOrgTrab();
        }
        $NRORG          = $this->environment->getNrOrgTrab();
        $CDOPERULTATU   = $this->environment->getCdOperador();
        $DTULTATU       = $currentTime->format("d/m/Y");
        $NRORGULTATU    = $this->environment->getNrOrgTrab();
        $NMTABEPREC     = $row[0]['NMTABEPREC'];
        $CDTABEPREC     = $row[0]['CDTABEPREC'];
        $CDFILIAL       = $row[0]['CDFILIAL'];
        if($isNew){
            $this->TabelaPrecoService->insertTabelaPreco($CDFILIAL,$CDTABEPREC,$NMTABEPREC,$IDATIVO,$CDOPERULTATU,$DTULTATU,$NRORGULTATU,$CDOPERINCLUSAO,$DTINCLUSAO,$NRORG,$NRORGINCLUSAO);
            $response->addNotification(new Notification("Tabela de preço {$NMTABEPREC} adicionada com sucesso!", Notification::TYPE_SUCCESS));
        } else
           parent::save($request, $response);
    }
    
    public function deleteTabelaPreco(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = "vnd42400_tabelapreco";
        $row = $request->getDataSet()->getRows();
        $NMTABEPREC     = $row[0]['NMTABEPREC'];
        $CDFILIAL       = $row[0]['CDFILIAL'];
        $CDTABEPREC     = $row[0]['CDTABEPREC'];
        $NRORG          = $this->environment->getNrOrgTrab();
        $isParameterized = $this->TabelaPrecoService->isParameterizedBySomething($CDFILIAL,$CDTABEPREC);
        if($isParameterized == 1){
            $response->setError(new DTO\Response\Error('Tabela de Preço não poderá ser excluida. Esta Tabela de Preço está parametrizada por Cliente Filial.',1));
        } elseif($isParameterized == 2){
            $response->setError(new DTO\Response\Error('Tabela de Preço não poderá ser excluida. Esta Tabela de Preço está parametrizada por Loja.',2));
        } elseif($isParameterized == 3){
            $response->setError(new DTO\Response\Error('Tabela de Preço não poderá ser excluida. Esta Tabela de Preço está parametrizada por Filial.',3));
        } elseif($isParameterized == 4){
            $response->setError(new DTO\Response\Error('Tabela de Preço não poderá ser excluida. Esta Tabela de Preço está parametrizada por Cliente.',4));
        } else{
            parent::delete($request, $response);
        }
    }
    
    public function savePrecoDiferenciado(DTO\Request $request, DTO\Response $response){
        $rows = $request->getDataSet()->getRows();
        foreach($rows as $key => &$row){
            $rows[$key]['HRFINPRECDIA'] = str_replace(':','',$rows[$key]['HRFINPRECDIA']);
            $rows[$key]['HRINIPRECDIA'] = str_replace(':','',$rows[$key]['HRINIPRECDIA']);
            $rows[$key]['VRPRECODIA'] = $this->valorBD($rows[$key]['VRPRECODIA']);
            $rows[$key]['VRPRECODIF'] = ($rows[$key]['VRPRECODIF']);
            unset($rows[$key]['DTFINVGPREC']);
        }
        if($rows[0]['__is_new']){
            $diasSemana = $rows[0]['NRDIASEMANPR'];
            $this->dataSourceName = "vnd42400_tabelapreco_precodiferenciado";
            foreach($diasSemana as $dia){
                $row->set('NRDIASEMANPR',$dia);
                if(!$this->TabelaPrecoService->verificaPrecoDifConsumidor($row)){
                    throw new \Exception("Tipo de consumidor não pode repetir no mesmo dia em um mesmo horário inicial ");
                }
                parent::save($request,$response);
            }
        } else {
            try{
                
                $currentTime = new \DateTime(null,new \DateTimeZone('America/Sao_Paulo'));

                $CDFILIAL       =   $row['CDFILIAL'];
                $CDTABEPREC     =   $row['CDTABEPREC'];
                $DTINIVGPREC    =   $row['DTINIVGPREC'];
                $CDPRODUTO      =   $row['CDPRODUTO'];
                $CDPRPAITABPR   =   $row['CDPRPAITABPR'];
                $NRDIASEMANPR   =   $row['NRDIASEMANPR'];
                $HRINIPRECDIA   =   $row['HRINIPRECDIA'];
                $HRFINPRECDIA   =   $row['HRFINPRECDIA'];
                $VRPRECODIA     =   $row['VRPRECODIA'];
                $IDPERVALORPR   =   $row['IDPERVALORPR'];
                $IDDESCACREPR   =   $row['IDDESCACREPR'];
                $DTINIVALPREC   =   $row['DTINIVALPREC'];
                $DTFINVALPREC   =   $row['DTFINVALPREC'];
                $CDTIPOCONSPD   =   $row['CDTIPOCONSPD'];
                $IDVISUACUPOM   =   $row['IDVISUACUPOM'];
                $NMTIPOCONS     =   $row['NMTIPOCONS'];
                $VRPRECODIF     =   $row['VRPRECODIF'];
                $NRORG          =   $this->environment->getNrOrgTrab();
                $CDOPERULTATU   =   $this->environment->getCdOperador();
                $DTULTATU       =   $currentTime->format("d/m/Y");
                $NRORGULTATU    =   $this->environment->getNrOrgTrab();
                
                $this->TabelaPrecoService->updatePrecoDiferenciado($CDFILIAL,$CDTABEPREC,$DTINIVGPREC,$CDPRODUTO,$CDPRPAITABPR,$NRDIASEMANPR,$HRINIPRECDIA,$HRFINPRECDIA,$VRPRECODIA,$IDPERVALORPR,$IDDESCACREPR,$DTINIVALPREC,$DTFINVALPREC,$CDTIPOCONSPD,$IDVISUACUPOM,$NMTIPOCONS,$NRORG,$CDOPERULTATU,$DTULTATU,$NRORGULTATU, $VRPRECODIF);
                
            } catch(\Exception $e) {
                $response->setError(new DTO\Response\Error($e->getMessage(), 2));
            }
        }
    }
    
    public function deletePrecoDiferenciado(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = "vnd42400_tabelapreco_precodiferenciado";
        if(method_exists($request,'getRow')){
            $request->getRow()->set('HRINIPRECDIA',str_replace(':','',$request->getRow()->get('HRINIPRECDIA')));
        }
        parent::delete($request,$response);
    }
    
    public function saveVigencia(DTO\Request $request, DTO\Response $response){
        $row = $request->getDataSet()->getRows();
        foreach($request->getDataSet()->getRows() as $key => &$row){
            $row['IDSITVGPREC'] = '1';
        }
        $DTINIVGPREC    = $row['DTINIVGPREC'];
        $DTFINVGPREC    = $row['DTFINVGPREC'];
        $NMTABEPREC     = $row['NMTABEPREC']; 
        
        $DTINIVG = strtotime(str_replace("/", "-", $DTINIVGPREC));
        $DTFINVG = strtotime(str_replace("/", "-", $DTFINVGPREC));
        
        if($DTINIVG > $DTFINVG){
            $response->addMessage(new Message("Data inicial não pode ser maior que data final de vigência.",Message::TYPE_ERROR));
            $response->setError(new DTO\Response\Error('Data inicial não pode ser maior que data final de vigência',1));
        } else{
            $this->dataSourceName = 'vnd42400_tabelapreco_vigencias';
            $CDFILIAL       = $row['CDFILIAL'];
            $CDTABEPREC     = $row['CDTABEPREC'];
            $currentTime = new \DateTime(null,new \DateTimeZone('America/Sao_Paulo'));
            if(!$row['GROUPED_LABEL'] || $row['__is_new']){  
               $VigenciaDateInterval = $this->TabelaPrecoService->getInsertValidVigenciaDate($CDFILIAL,$CDTABEPREC);
            } else{
                if(strlen($row['OLD_DTINIVGPREC'])>10)
                    $OLD_DTINIVGPREC = $row['OLD_DTINIVGPREC'];
                else 
                    $OLD_DTINIVGPREC = $row['OLD_DTINIVGPREC'];
                if(strlen($row['OLD_DTFINVGPREC'])>10)    
                    $OLD_DTFINVGPREC = $row['OLD_DTFINVGPREC'];
                else 
                    $OLD_DTFINVGPREC = $row['OLD_DTFINVGPREC'];
                
                $VigenciaDateInterval = $this->TabelaPrecoService->getUpdateValidVigenciaDate($CDFILIAL,$CDTABEPREC,$OLD_DTINIVGPREC,$OLD_DTFINVGPREC);
            }
            if(count($VigenciaDateInterval)>2){
                $MaxVigencia = end($VigenciaDateInterval);
            } else{
                $MaxVigencia = null;
            }
            foreach($VigenciaDateInterval as $key => $VigenciaDate){
                $INICIO =   date_create_from_format('d/m/Y h:i:s',$VigenciaDate['INICIO']);
                $FIM    =   date_create_from_format('d/m/Y h:i:s',$VigenciaDate['FIM']);
                if(($DTINIVGPREC > $INICIO) && ($DTINIVGPREC < $FIM) && ($VigenciaDate != $MaxVigencia)){
                    $response->addMessage(new Message("Não é possível salvar. Data inicial encontra-se entre uma vigência existente.",Message::TYPE_ERROR)); 
                    
                    return $response;
                } elseif(($DTFINVGPREC > $INICIO) && ($DTFINVGPREC < $FIM) && ($VigenciaDate != $MaxVigencia)){
                    $response->addMessage(new Message("Não é possível salvar. Data final encontra-se entre uma vigência existente.",Message::TYPE_ERROR)); 
                    
                    return $response;
                } elseif(($DTINIVGPREC < $INICIO) && ($DTFINVGPREC > $FIM)){
                    $response->addMessage(new Message("Não é possível salvar. Existe uma vigência entre as datas informadas.",Message::TYPE_ERROR)); 
                    
                    return $response;
                } 
            }
            
            if(!$row['GROUPED_LABEL']){
                $CDOPERINCLUSAO = $this->environment->getCdOperador();
                $DTINCLUSAO     = $currentTime->format("d/m/Y");
                $NRORGINCLUSAO  = $this->environment->getNrOrgTrab();
            }
            
            $NRORG          = $this->environment->getNrOrgTrab();
            $NRORGINCLUSAO          = $this->environment->getNrOrgTrab();
            $CDOPERULTATU   = $this->environment->getCdOperador();
            $CDOPERINCLUSAO   = $this->environment->getCdOperador();
            $DTULTATU       = $currentTime->format("d/m/Y");
            $DTINCLUSAO       = $currentTime->format("d/m/Y");
            $NRORGULTATU    = $this->environment->getNrOrgTrab();
            if(!$row['GROUPED_LABEL'] || $row['__is_new']){
                $this->TabelaPrecoService->InsertVigencia($CDFILIAL,$CDTABEPREC,$DTINIVGPREC, $DTFINVGPREC, $CDOPERINCLUSAO,$DTINCLUSAO,$NRORGINCLUSAO,$NRORG,$CDOPERULTATU,$DTULTATU,$NRORGULTATU);
                $this ->TabelaPrecoService->updateTABEPREC($NMTABEPREC,$NRORG,$CDOPERULTATU,$DTULTATU,$NRORGULTATU,$CDTABEPREC);

            } else{
                $this->TabelaPrecoService->updateVigencia($CDFILIAL,$CDTABEPREC,$DTINIVGPREC, $DTFINVGPREC, $OLD_DTINIVGPREC, $OLD_DTFINVGPREC ,$NRORG,$CDOPERULTATU,$DTULTATU,$NRORGULTATU);
               $this ->TabelaPrecoService->updateTABEPREC($NMTABEPREC,$NRORG,$CDOPERULTATU,$DTULTATU,$NRORGULTATU,$CDTABEPREC);

            }
        }
    }
        
        
    public function saveProduto(DTO\Request $request, DTO\Response $response){
        $row = $request->getDataSet()->getRows();
        $currentTime = new \DateTime(null,new \DateTimeZone('America/Sao_Paulo'));
        $isNew = $row[0]['__is_new'];
        $params = array(
            'IDATIVO' => 'S',
            'NRORG'          => $this->environment->getNrOrgTrab(),
            'CDOPERULTATU'   => $this->environment->getCdOperador(),
            'DTULTATU'       => $currentTime->format("d/m/Y"),
            'NRORGULTATU'    => $this->environment->getNrOrgTrab(),
            'CDFILIAL'       => $row[0]['CDFILIAL'],
            'CDTABEPREC'     => $row[0]['CDTABEPREC'],
            'DTINIVGPREC'    => $row[0]['DTINIVGPREC'],
            'VRCUSTITEM'     => $row[0]['VRCUSTITEM']   = Util::moneyToDouble($row[0]["VRCUSTITEM"]),
            'VRPRECITEM'     => $row[0]["VRPRECITEM"]   = Util::moneyToDouble($row[0]["VRPRECITEM"]),
            'VRPRESUGITEM'   => $row[0]["VRPRESUGITEM"] = Util::moneyToDouble($row[0]["VRPRESUGITEM"]),
            'VRPRECITEMCL'   => $row[0]["VRPRECITEMCL"] = Util::moneyToDouble($row[0]["VRPRECITEMCL"]),
            'IDPRECVARIA'    => $row[0]["IDPRECVARIA"] ? $row[0]["IDPRECVARIA"] : 'N',
            'IDPRODPROMOC'   => $row[0]["IDPRODPROMOC"] ? $row[0]["IDPRODPROMOC"] : 'N',
            'HRFIMVENPROD'       => $row[0]['HRFIMVENPROD'],
            'HRINIVENPROD'       => $row[0]['HRINIVENPROD']
        );
        if($isNew){
            $CDPRODUTOINI    = $row[0]['CDARVPRODINI'];
            if(!isset($row[0]['CDARVPRODFIN'])){
                $CDPRODUTOFIN    = $CDPRODUTOINI;
            }else{
                $CDPRODUTOFIN    = $row[0]['CDARVPRODFIN'];
            }
            $params = array_merge($params,array(
                'CDPRODUTOINI' => $CDPRODUTOINI,
                'CDPRODUTOFIN' => $CDPRODUTOFIN,
                'CDOPERINCLUSAO'=> $this->environment->getCdOperador(),
                'DTINCLUSAO'=> $currentTime->format("d/m/Y"),
                'NRORGINCLUSAO' => $this->environment->getNrOrgTrab()
            ));
            $data = $row[0]["DATA"];
            try{
                $result = $this->TabelaPrecoService->insertPrecoItem($params,$data);
                $response->addDataSet(new DataSet('isPlural', array('result' => $result)));
            }catch(Exception $e){
                $response->setError(new Error($e->getMessage(), 500));
            }
        }else{
            $CDPRODUTO = $CDPRPAITABPR = $row[0]['CDPRODUTO'];
            $params = array_merge($params,array(
                'CDPRODUTO' => $CDPRODUTO,
                'CDPRPAITABPR' => $CDPRPAITABPR
            ));
            try{
                $this->TabelaPrecoService->updatePrecoItem($params);
                $response->addDataSet(new DataSet('isPlural', array('result' => 1)));
            }catch(Exception $e){
                $response->setError(new Error($e->getMessage(), 500));
            }
        }
    }
    
    public function saveProdutoInline(DTO\Request $request, DTO\Response $response) {
        try {
            foreach($request->getRow() as $produto){
                $this->TabelaPrecoService->updatePrecoItemInline($produto['CDFILIAL'],$produto['NRORG'],$produto['CDTABEPREC'],$produto['DTINIVGPREC'],$produto['CDPRODUTO'],$produto['VRPRECITEM'],$produto['VRCUSTITEM'],$produto['VRPRESUGITEM'],$produto['VRPRECITEMCL'], $produto['IDPRODPROMOC'], $produto['HRINIVENPROD'], $produto['HRFIMVENPROD']);
                
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function saveProdutoInlinePrecoDiferenciado(DTO\Request $request, DTO\Response $response) {
        try {
            foreach($request->getRow() as $produto){
                $produto['HRINIPRECDIA'] = str_replace(':', '', $produto['HRINIPRECDIA']);
                $produto['VRPRECODIA'] = str_replace(',', '.', $produto['VRPRECODIA']);
                $params = array(
                    ':VRPRECODIA'   =>  $produto['VRPRECODIA'],
                    ':CDTABEPREC'   =>  $produto['CDTABEPREC'],
                    ':DTINIVGPREC'  =>  $produto['DTINIVGPREC'],
                    ':CDPRODUTO'    =>  $produto['CDPRODUTO'],
                    ':NRORG'        =>  $produto['NRORG'],
                    ':CDFILIAL'     =>  $produto['CDFILIAL'],
                    ':CDPRPAITABPR' =>  $produto['CDPRPAITABPR'],
                    ':HRINIPRECDIA' =>  $produto['HRINIPRECDIA'],
                    ':CDTIPOCONSPD' =>  $produto['CDTIPOCONSPD'],
                    ':CDPRPAITABPR' =>  $produto['CDPRPAITABPR'],
                    ':NRDIASEMANPR' =>  $produto['NRDIASEMANPR']
                ); 
                
                $this->TabelaPrecoService->updatePrecoItemInlinePrecoDiferenciado($params);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
   
    public function deletePrecoDiferenciadoDataSet(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = "vnd42400_tabelapreco_precodiferenciado";
        parent::delete($request,$response);
    }
    
    public function atualizaPrecos(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        $params = array(
            ':CDFILIAL'    => $row['CDFILIAL'],
            ':CDTABEPREC'  => $row['CDTABEPREC'],
            ':DTINIVGPREC' => substr($row['DTINIVGPREC'], 0, 10)
        );
        
        $this->TabelaPrecoService->updItemPrecoAtuPrec($params);
    }
    
    public function alteraPrecoAutomaticamente(DTO\Request\Row $request, DTO\Response $response) {
        $row = $this->util->getParamsAsArray($request);

        $params = array(
            ':CDTABEPREC'  => $row['CDTABEPREC'],
            ':CDFILIAL'    => $row['CDFILIAL'],
            ':DTINIVGPREC' => $row['DTINIVGPREC'],
            ':NRORG'       => $row['NRORG']
        );
        
        if($row['TIPOPROD'] == 'I') {
            $params[':CDPRODUTOINI'] = $row['CDPRODUTOINI'];
            $params[':CDPRODUTOFIN'] = $row['CDPRODUTOFIN'];
            
            $intervalo = $this->TabelaPrecoService->getIntervaloCadastroAutomatico($params);
            
            foreach($intervalo as &$produto) {
                $auxIntervalo[] = $produto['CDPRODUTO'];
            }
            
            $intervalo = $auxIntervalo;
        } else {
            $intervalo = $row['PRODUTOS'];
        }
        
        //retira o ponto da máscara
        $row['VRPRECITEM'] = str_replace('.', '', $row['VRPRECITEM']);
        $row['VRPRECITEM'] = str_replace(',', '.', $row['VRPRECITEM']);
        $params['VRPRECITEM'] = (float)$row['VRPRECITEM'];
        
        for($i = 0; $i < count($intervalo); $i++) {
            $params['CDPRODUTO'] = $intervalo[$i];
            
            $this->TabelaPrecoService->setPrecoAutomatico($params);
        }
    }
    
    public function deleteProduto(DTO\Request\Row $request, DTO\Response $response) {
        $row = $this->util->getParamsAsArray($request);
        
        $params = array(
            ':NRORG'       => $row['NRORG'],
            ':CDFILIAL'    => $row['CDFILIAL'],
            ':CDTABEPREC'  => $row['CDTABEPREC'],
            ':CDPRODUTO'   => $row['CDPRODUTO'],
            ':DTINIVGPREC' => $row['DTINIVGPREC']
        );
        
        $this->TabelaPrecoService->deletePreco($params);
        $this->TabelaPrecoService->deletePrecosDiferenciados($params);
    }
    
    public function calcularPrecos(DTO\Request\Row $request, DTO\Response $response) {
        $row = $this->util->getParamsAsArray($request);
        
        $row['NRORG'] = $this->environment->getNrOrgTrab();

        $dataset = $this->calcularPrecosService->calcularPreco($row);
        
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('vnd42400_tabelapreco_calcularPrecos', $dataset));
    }
    
    private function valorBD($valorAplicado){
		$posVirgula = strpos( $valorAplicado, "," );
		$posPont = strpos( $valorAplicado, "." );
		
		if (!$posVirgula === false && !$posPont === false) {
			$valor = str_replace("." , "" , $valorAplicado ); // Primeiro tira os pontos
			$valorAplicado2 = str_replace(',','.',$valor);
		}else if(!$posVirgula === false && $posPont === false){
			$valorAplicado2 = str_replace(',','.',$valorAplicado);
		}else {
			$valorAplicado2 = $valorAplicado;
		}
		return $valorAplicado2;
	}
	
    public function manutencaoPrecos(DTO\Request\Row $request, DTO\Response $response){
        try{
            $row = $this->util->getParamsAsArray($request);
            $currentTime = new \DateTime(null,new \DateTimeZone('America/Sao_Paulo'));
            foreach($row as $key => $value){
                $row[$key]['NRORGBACK'] = $this->environment->getNrOrgTrab();
                $row[$key]['NRORGINCLUSAO'] = $this->environment->getNrOrgTrab();
                $row[$key]['CDOPERULTATU'] = $this->environment->getCdOperador();
                $row[$key]['CDOPERINCLUSAO'] = $this->environment->getCdOperador();
                $row[$key]['DTULTATU'] = $currentTime->format("d/m/Y");
                $row[$key]['INCLUSAODT'] = $currentTime->format("d/m/Y");
                $row[$key]['DTALTERACAO'] = $currentTime->format("d/m/Y");
                $row[$key]['NRORGULTATU'] = $this->environment->getNrOrgTrab();
                $row[$key]['CDOPERADOR'] = $this->environment->getCdOperador();
            }
        
            $res = $this->TabelaPrecoService->manutencaoPrecos($row);
            
            self::escreverLog($res);
            
            $array = array(
                "status" => true,
                "log" => $this->getLog(),
                'message' => "Operação realizada com sucesso!"
            );
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('resultado', $array));
            
        } catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
    }
}