<?php

namespace Teknisa\MAN\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\NewCode;
use Teknisa\MAN\Utilities;


class RelDREComparativoService {

	public function __construct(EntityManager $entityManager, newCode $newCode) {
        $this->entityManager = $entityManager;
        $this->newCode = $newCode;
    }
    
    /********************
     *  REPORT METHODS  *
     ********************/

    public function unidadeParamBuild($conditions){
        $inimes = '01'.'/'.explode('/', $conditions['DATAINI'])[0].'/'.(intval(explode('/', $conditions['DATAINI'])[1]));
        $finmes = date('t', strtotime(str_replace("/", "-", $inimes))).'/'.explode('/', $conditions['DATAINI'])[0].'/'.(intval(explode('/', $conditions['DATAINI'])[1]));
        $filialParams = self::buildFilialParams($conditions);
        $movimentacoesEst = self::buildMoviEstParams($conditions);
        $params = array (
            'REGIME'    => $conditions['REGIME'],
            'NRORG'     => $conditions['NRORG'],
            'DTINI'     => $inimes,
            'DTFIN'     => $finmes,
            'CDFILIAL'  => $filialParams['CDFILIAL'],
            'CDFILIAL1' => $filialParams['CDFILIAL1'],
            'CDFILIAL2' => $filialParams['CDFILIAL2'],
            'CDFILIAL3' => $filialParams['CDFILIAL3'],
            'CDFILIAL4' => $filialParams['CDFILIAL4'],
            'CDFILIAL5' => $filialParams['CDFILIAL5'],
            'CDFILIAL6' => $filialParams['CDFILIAL6'],
            'MOVIEST1'  => $movimentacoesEst['MOVIEST1'],
            'MOVIEST2'  => $movimentacoesEst['MOVIEST2'],
            'MOVIEST3'  => $movimentacoesEst['MOVIEST3'],
            'MOVIEST4'  => $movimentacoesEst['MOVIEST4'],
            'MOVIEST5'  => $movimentacoesEst['MOVIEST5'],
            'MOVIEST6'  => $movimentacoesEst['MOVIEST6']
        );
        $reportName = 'DREC_Unidade';
        $params = array(
            'ParametersRow' => $params,
            'ReportName'    => $reportName
        );
        return $params;
    }
    
    public function mensalParamBuild($conditions){
        $dateParams       = self::buildDateParams($conditions);
        $movimentacoesEst = self::buildMoviEstParams($conditions);
        $params = array (
            'CDFILIAL' => $conditions['CDFILIAL'],
            'REGIME'   => $conditions['REGIME'],
            'NRORG'    => $conditions['NRORG'],
            'INIMES1'  => $dateParams['INIMES1'],
            'FINMES1'  => $dateParams['FINMES1'],
            'INIMES2'  => $dateParams['INIMES2'],
            'FINMES2'  => $dateParams['FINMES2'],
            'INIMES3'  => $dateParams['INIMES3'],
            'FINMES3'  => $dateParams['FINMES3'],
            'INIMES4'  => $dateParams['INIMES4'],
            'FINMES4'  => $dateParams['FINMES4'],
            'INIMES5'  => $dateParams['INIMES5'],
            'FINMES5'  => $dateParams['FINMES5'],
            'INIMES6'  => $dateParams['INIMES6'],
            'FINMES6'  => $dateParams['FINMES6'],
            'MOVIEST1' => $movimentacoesEst['MOVIEST1'],
            'MOVIEST2' => $movimentacoesEst['MOVIEST2'],
            'MOVIEST3' => $movimentacoesEst['MOVIEST3'],
            'MOVIEST4' => $movimentacoesEst['MOVIEST4'],
            'MOVIEST5' => $movimentacoesEst['MOVIEST5'],
            'MOVIEST6' => $movimentacoesEst['MOVIEST6']
        );
        $reportName = 'DREC_Mensal';
        $params = array(
            'ParametersRow' => $params,
            'ReportName'    => $reportName
        );
        return $params;
    }
    
    private function buildFilialParams($conditions){
        $filial1 = array_key_exists('CDFILIAL', $conditions)   ? $conditions['CDFILIAL']  : 'AAAA';
        $filial2 = (array_key_exists('CDFILIAL2', $conditions) && $conditions['CDFILIAL2'] != "")  ? $conditions['CDFILIAL2'] : 'AAAA';
        $filial3 = array_key_exists('CDFILIAL3', $conditions)  ? $conditions['CDFILIAL3'] : 'AAAA';
        $filial4 = array_key_exists('CDFILIAL4', $conditions)  ? $conditions['CDFILIAL4'] : 'AAAA';
        $filial5 = array_key_exists('CDFILIAL5', $conditions)  ? $conditions['CDFILIAL5'] : 'AAAA';
        $filial6 = array_key_exists('CDFILIAL6', $conditions)  ? $conditions['CDFILIAL6'] : 'AAAA';
        $filiais = $filial1.'_'.$filial2.'_'.$filial3.'_'.$filial4.'_'.$filial5.'_'.$filial6;
        $data = array(
            'CDFILIAL'  => $filiais,
            'CDFILIAL1' => $filial1,
            'CDFILIAL2' => $filial2,
            'CDFILIAL3' => $filial3,
            'CDFILIAL4' => $filial4,
            'CDFILIAL5' => $filial5,
            'CDFILIAL6' => $filial6
        );
        return $data;
    }
        
    
    
    private function buildDateParams($conditions){
        if(array_key_exists('MES1', $conditions)  && $conditions['MES1'] != ""){
            $inimes1 = '01'.'/'.explode('/', $conditions['MES1'])[0].'/'.(intval(explode('/', $conditions['MES1'])[1]));
            $finmes1 = date('t', strtotime(str_replace("/", "-", $inimes1))).'/'.explode('/', $conditions['MES1'])[0].'/'.(intval(explode('/', $conditions['MES1'])[1]));
        }
        if(array_key_exists('MES2', $conditions) && $conditions['MES2'] != ""){
            $inimes2 = '01'.'/'.explode('/', $conditions['MES2'])[0].'/'.(intval(explode('/', $conditions['MES2'])[1]));
            $finmes2 = date('t', strtotime(str_replace("/", "-", $inimes2))).'/'.explode('/', $conditions['MES2'])[0].'/'.(intval(explode('/', $conditions['MES2'])[1]));
        }
        if(array_key_exists('MES3', $conditions) && $conditions['MES3'] != ""){
            $inimes3 = '01'.'/'.explode('/', $conditions['MES3'])[0].'/'.(intval(explode('/', $conditions['MES3'])[1]));
            $finmes3 = date('t', strtotime(str_replace("/", "-", $inimes3))).'/'.explode('/', $conditions['MES3'])[0].'/'.(intval(explode('/', $conditions['MES3'])[1]));
        }
        if(array_key_exists('MES4', $conditions) && $conditions['MES4'] != ""){
            $inimes4 = '01'.'/'.explode('/', $conditions['MES4'])[0].'/'.(intval(explode('/', $conditions['MES4'])[1]));
            $finmes4 = date('t', strtotime(str_replace("/", "-", $inimes4))).'/'.explode('/', $conditions['MES4'])[0].'/'.(intval(explode('/', $conditions['MES4'])[1]));
        }
        if(array_key_exists('MES5', $conditions) && $conditions['MES5'] != ""){
            $inimes5 = '01'.'/'.explode('/', $conditions['MES5'])[0].'/'.(intval(explode('/', $conditions['MES5'])[1]));
            $finmes5 = date('t', strtotime(str_replace("/", "-", $inimes5))).'/'.explode('/', $conditions['MES5'])[0].'/'.(intval(explode('/', $conditions['MES5'])[1]));
        }
        if(array_key_exists('MES6', $conditions) && $conditions['MES6'] != ""){
            $inimes6 = '01'.'/'.explode('/', $conditions['MES6'])[0].'/'.(intval(explode('/', $conditions['MES6'])[1]));
            $finmes6 = date('t', strtotime(str_replace("/", "-", $inimes6))).'/'.explode('/', $conditions['MES6'])[0].'/'.(intval(explode('/', $conditions['MES6'])[1]));
        }
        //Foi necessário inverter a ordem de envio dos parâmetros de data para as queries
        //para dessa forma poder exibir os dados no relatório em ordem correta(da direita pra esquerda)
        $dateParams = array(
            'INIMES1' => isset($inimes6) ? $inimes6 : '01/01/1800',
            'FINMES1' => isset($finmes6) ? $finmes6 : '01/01/1800',
            'INIMES2' => isset($inimes5) ? $inimes5 : '01/01/1800',
            'FINMES2' => isset($finmes5) ? $finmes5 : '01/01/1800',
            'INIMES3' => isset($inimes4) ? $inimes4 : '01/01/1800',
            'FINMES3' => isset($finmes4) ? $finmes4 : '01/01/1800',
            'INIMES4' => isset($inimes3) ? $inimes3 : '01/01/1800',
            'FINMES4' => isset($finmes3) ? $finmes3 : '01/01/1800',
            'INIMES5' => isset($inimes2) ? $inimes2 : '01/01/1800',
            'FINMES5' => isset($finmes2) ? $finmes2 : '01/01/1800',
            'INIMES6' => isset($inimes1) ? $inimes1 : '01/01/1800',
            'FINMES6' => isset($finmes1) ? $finmes1 : '01/01/1800'
            );
        return $dateParams;
    }
    
    private function buildMoviEstParams($conditions){
        $moviEst1 = $this->newCode->getCodeByTableName('MOVIESTOQUE', $conditions['NRORG']);
        $moviEst2 = $this->newCode->getCodeByTableName('MOVIESTOQUE', $conditions['NRORG']);
        $moviEst3 = $this->newCode->getCodeByTableName('MOVIESTOQUE', $conditions['NRORG']);
        $moviEst4 = $this->newCode->getCodeByTableName('MOVIESTOQUE', $conditions['NRORG']);
        $moviEst5 = $this->newCode->getCodeByTableName('MOVIESTOQUE', $conditions['NRORG']);
        $moviEst6 = $this->newCode->getCodeByTableName('MOVIESTOQUE', $conditions['NRORG']);
        $moviEstoque = array(
            'MOVIEST1'  => $moviEst1,    
            'MOVIEST2'  => $moviEst2,    
            'MOVIEST3'  => $moviEst3,    
            'MOVIEST4'  => $moviEst4,    
            'MOVIEST5'  => $moviEst5,    
            'MOVIEST6'  => $moviEst6    
        );
        return $moviEstoque;
    }
   
}